using LeedsBeerQuest.API.Data.Contexts;
using LeedsBeerQuest.API.Data.Models;
using LeedsBeerQuest.API.Data.Services.Interfaces;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace LeedsBeerQuest.API.Controllers;

[ApiController]
[Route("[controller]")]
public class VenueController : ControllerBase
{
    private readonly IVenueService _venueService;

    public VenueController(IVenueService venueService)
    {
        _venueService = venueService;
    }

    [HttpGet]
    public async Task<IActionResult> GetAll()
    {
        var venues = await _venueService.GetAllVenues();
        return Ok(venues);
    }

    [HttpGet("{venueId:int}")]
    public async Task<IActionResult> GetVenue(int venueId)
    {
        if (venueId < 1)
        {
            return BadRequest("Requested venue ID must be more than zero");
        }

        var venue = await _venueService.GetVenue(venueId);

        return venue == null ? NotFound() : Ok(venue);
    }
}